"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const plugin_name_1 = require("./plugin.name");
/**
 * @internal
 */
class BabelMultiTargetHtmlUpdater {
    constructor(compilationTargets) {
        this.compilationTargets = compilationTargets;
    }
    addAssetsFromChildCompilations(compilation, htmlPluginData) {
        compilation.children
            .filter((child) => {
            const isMultiTargetChild = child.name && child.name.startsWith(plugin_name_1.CHILD_COMPILER_PREFIX);
            if (!isMultiTargetChild) {
                return false;
            }
            const target = this.compilationTargets[child.name];
            return target.esModule || target.noModule;
        })
            .forEach((child) => {
            const jsChunkGroups = child.chunkGroups
                .filter((chunkGroup) => {
                // webpack doesn't export Entrypoint :/
                return chunkGroup.constructor.name === 'Entrypoint' &&
                    chunkGroup.chunks.find(chunk => !!chunk.files.find(file => file.endsWith('.js')));
            });
            const jsChunks = jsChunkGroups
                .reduce((result, group) => {
                result.push(...group.chunks);
                return result;
            }, []);
            // the plugin already sorted the chunks from the main compilation,
            // so we'll need to do it for the children as well
            let sortedChunks = htmlPluginData.plugin.sortChunks(jsChunks, htmlPluginData.plugin.options.chunksSortMode, jsChunkGroups);
            // generate the chunk objects used by the plugin
            const htmlChunks = sortedChunks.reduce((result, chunk) => {
                let entry = chunk.files.find((file) => file.endsWith('.js'));
                result[chunk.name] = {
                    css: [],
                    entry,
                    hash: chunk.hash,
                };
            }, {});
            Object.assign(htmlPluginData.assets.chunks, htmlChunks);
            // add the asset names form the child
            let assetNames = sortedChunks.map(chunk => chunk.files.find((file) => file.endsWith('.js')));
            htmlPluginData.assets.js.push(...assetNames);
        });
    }
    updateScriptTags(compilation, htmlPluginData) {
        let childCompilations = compilation.children
            .filter(child => child.name.startsWith(plugin_name_1.CHILD_COMPILER_PREFIX));
        htmlPluginData.head
            .concat(htmlPluginData.body)
            .filter((tag) => tag.tagName === 'script')
            .forEach((tag) => {
            const childCompilation = childCompilations
                .find(child => !!child.assets[tag.attributes.src]);
            const target = this.compilationTargets[childCompilation.name];
            if (target.esModule) {
                tag.attributes.type = 'module';
            }
            else {
                tag.attributes.nomodule = true;
            }
        });
    }
    apply(compiler) {
        compiler.hooks.compilation.tap(plugin_name_1.PLUGIN_NAME, (compilation) => {
            compilation.hooks.htmlWebpackPluginBeforeHtmlGeneration.tapPromise(plugin_name_1.PLUGIN_NAME, (htmlPluginData) => __awaiter(this, void 0, void 0, function* () {
                this.addAssetsFromChildCompilations(compilation, htmlPluginData);
                return htmlPluginData;
            }));
            compilation.hooks.htmlWebpackPluginAlterAssetTags.tapPromise(plugin_name_1.PLUGIN_NAME, (htmlPluginData) => __awaiter(this, void 0, void 0, function* () {
                this.updateScriptTags(compilation, htmlPluginData);
                return htmlPluginData;
            }));
        });
    }
}
exports.BabelMultiTargetHtmlUpdater = BabelMultiTargetHtmlUpdater;
//# sourceMappingURL=babel.multi.target.html.updater.js.map