"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const defaults_1 = require("./defaults");
const transpiler_compiler_1 = require("./transpiler.compiler");
class BabelMultiTargetPlugin {
    constructor(options) {
        if (options.plugins && typeof (options.plugins) !== 'function') {
            throw new Error('WebpackBabelMultiTargetOptions.plugins must be a function');
        }
        if (!options.babel) {
            options.babel = {};
        }
        if (!options.babel.plugins) {
            options.babel.plugins = [];
        }
        if (!options.babel.presetOptions) {
            options.babel.presetOptions = {};
        }
        if (!options.targets) {
            options.targets = defaults_1.DEFAULT_TARGET_INFO;
        }
        if (!options.exclude) {
            options.exclude = [];
        }
        this.options = options;
        this.targets = Object.keys(options.targets)
            .reduce((result, profileName) => {
            const targetInfo = options.targets[profileName];
            const browsers = targetInfo.browsers || defaults_1.DEFAULT_BROWSERS[profileName];
            const key = targetInfo.key || profileName;
            result.push(Object.assign({}, defaults_1.DEFAULT_TARGET_INFO[profileName], targetInfo, {
                profileName,
                browsers,
                key,
                options: this.createTransformOptions(browsers),
            }));
            return result;
        }, []);
        if (!this.targets.length) {
            throw new Error('Must provide at least one target');
        }
        if (this.targets.filter(target => target.tagAssetsWithKey === false).length > 1) {
            throw new Error('Only one target may have the `tagAssetsWithKey` property set to `false`');
        }
        if (this.targets.filter(target => target.esModule).length > 1) {
            throw new Error('Only one target may have the `esModule` property set to `true`');
        }
        if (this.targets.filter(target => target.noModule).length > 1) {
            throw new Error('Only one target may have the `noModule` property set to `true`');
        }
        this.transpilerCompiler = new transpiler_compiler_1.TranspilerCompiler(this.targets, this.options.config, this.options.plugins, this.options.exclude);
    }
    createTransformOptions(browsers) {
        const mergedPresetOptions = Object.assign({}, defaults_1.DEFAULT_BABEL_PRESET_OPTIONS, this.options.babel.presetOptions, {
            targets: {
                browsers,
            },
        }, {
            modules: false,
        });
        return {
            presets: [
                ['@babel/preset-env', mergedPresetOptions],
            ],
            plugins: [
                ...defaults_1.DEFAULT_BABEL_PLUGINS,
                ...this.options.babel.plugins,
            ],
        };
    }
    apply(compiler) {
        this.transpilerCompiler.init(compiler);
    }
}
exports.BabelMultiTargetPlugin = BabelMultiTargetPlugin;
//# sourceMappingURL=babel.multi.target.plugin.js.map