"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BABEL_LOADER = 'babel-loader';
/**
 * @internal
 */
class BabelRuleConverter {
    convertLoader(loader, options) {
        const ogOptions = loader ? loader.options || loader.query : {};
        return {
            loader: exports.BABEL_LOADER,
            options: Object.assign(ogOptions, options),
        };
    }
    findAndConvertLoaders(ruleLoaders, options) {
        if (Array.isArray(ruleLoaders)) {
            const loaders = ruleLoaders;
            return loaders.reduce((result, loader) => {
                if (!this.isBabelLoader(loader)) {
                    result.loaders.push(loader);
                    return result;
                }
                result.loaders.push(this.convertLoader(loader, options));
                result.converted++;
                return result;
            }, { converted: 0, loaders: [] });
        }
        const loader = ruleLoaders;
        if (!this.isBabelLoader(loader)) {
            return {
                converted: 0,
                loaders: loader,
            };
        }
        return {
            converted: 1,
            loaders: this.convertLoader(loader, options),
        };
    }
    isBabelLoader(loader) {
        if (loader === exports.BABEL_LOADER) {
            return true;
        }
        return loader.loader === exports.BABEL_LOADER;
    }
    convertLoaders(rules, exclude, options) {
        return rules.reduce((result, rule) => {
            const loaderRule = rule;
            const useRule = rule;
            const loaders = loaderRule.loader || useRule.use;
            const loaderConversion = this.findAndConvertLoaders(loaders, options);
            useRule.use = loaderConversion.loaders;
            if (!useRule.exclude) {
                useRule.exclude = [];
            }
            else if (!Array.isArray(useRule.exclude)) {
                useRule.exclude = [useRule.exclude];
            }
            useRule.exclude.push(...exclude);
            result.rules.push(useRule);
            result.converted += loaderConversion.converted;
            return result;
        }, { converted: 0, rules: [] });
    }
}
exports.BabelRuleConverter = BabelRuleConverter;
//# sourceMappingURL=babel.rule.converter.js.map