"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const plugin_name_1 = require("./plugin.name");
/**
 * @internal
 */
class BabelTargetChunkIdUpdater {
    constructor(target) {
        this.target = target;
    }
    apply(compiler) {
        compiler.hooks.compilation.tap(plugin_name_1.PLUGIN_NAME, (compilation) => {
            // add the key as the chunk name suffix for any chunks created
            compilation.hooks.beforeChunkIds.tap(plugin_name_1.PLUGIN_NAME, (chunks) => {
                chunks.forEach(chunk => {
                    if (chunk.id || chunk.name) {
                        let id = chunk.id || chunk.name;
                        if (this.target.tagAssetsWithKey !== false) {
                            id += `.${this.target.key}`;
                        }
                        chunk.id = id;
                        chunk.name = id;
                    }
                });
            });
        });
    }
}
exports.BabelTargetChunkIdUpdater = BabelTargetChunkIdUpdater;
//# sourceMappingURL=babel.target.chunk.id.updater.js.map