"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = require("fs-extra");
const os = require("os");
const path = require("path");
/**
 * @internal
 */
class TempEmitter {
    constructor(compilation) {
        this.compilation = compilation;
        this.files = [];
        this._context = path.resolve(os.tmpdir(), compilation.fullHash);
    }
    get context() {
        return this._context;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDisposed) {
                throw new Error('Cannot init, already disposed');
            }
            yield fs_extra_1.mkdirp(this._context);
        });
    }
    emit() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDisposed) {
                throw new Error('Cannot emit, already disposed');
            }
            return yield Promise.all(Object.keys(this.compilation.assets)
                .map((assetFile) => __awaiter(this, void 0, void 0, function* () {
                const asset = this.compilation.assets[assetFile];
                const assetPath = path.resolve(this._context, assetFile);
                yield fs_extra_1.mkdirp(path.dirname(assetPath));
                yield fs_extra_1.writeFile(assetPath, asset.source(), 'utf-8');
                this.files.push(assetPath);
                return {
                    name: assetFile,
                    path: assetPath,
                };
            })));
        });
    }
    dispose() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDisposed) {
                return;
            }
            yield Promise.all(this.files.map(file => fs_extra_1.unlink(file)));
            this.files = undefined;
            yield fs_extra_1.rmdir(this._context);
            this._context = undefined;
            this.isDisposed = true;
        });
    }
}
exports.TempEmitter = TempEmitter;
//# sourceMappingURL=temp.emitter.js.map