"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const babel_multi_target_html_updater_1 = require("./babel.multi.target.html.updater");
const babel_target_compiler_factory_1 = require("./babel.target.compiler.factory");
const plugin_name_1 = require("./plugin.name");
const temp_emitter_1 = require("./temp.emitter");
/**
 * @internal
 */
class TranspilerCompiler {
    constructor(targets, config, plugins, exclude) {
        this.targets = targets;
        this.config = config;
        this.plugins = plugins;
        this.exclude = exclude;
        this.compilationTargets = {};
    }
    runChildCompilers(parent, childCompilers) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all(childCompilers.map(childCompiler => {
                childCompiler.parentCompilation = parent;
                return new Promise((resolve, reject) => childCompiler.runAsChild((err) => {
                    if (err) {
                        return reject(err);
                    }
                    resolve();
                }));
            }));
        });
    }
    cleanUpOriginalCompilation(compilation) {
        // if one of the targets is not being tagged with its key, it will overwrite the output of the original
        // compilation, and we don't need to do anything further
        if (this.targets.find(target => target.tagAssetsWithKey === false)) {
            return;
        }
        // if all target outputs are being tagged, we need to delete the output of the original compilation
        compilation.chunkGroups.forEach(group => {
            group.chunks.forEach(chunk => {
                const id = chunk.name || chunk.id;
                if (!id) {
                    return;
                }
                // remove the assets so they aren't emitted
                delete compilation.assets[`${id}.js`];
                delete compilation.assets[`${id}.js.map`];
                // remove js and js.map files so they aren't referenced by HtmlWebpackPlugin
                // this leaves any other files (like css) so they can still be referenced
                chunk.files = group.runtimeChunk.files.filter(file => !/\.js(?:\.map)?$/.test(file));
            });
        });
    }
    init(compiler) {
        new babel_multi_target_html_updater_1.BabelMultiTargetHtmlUpdater(this.compilationTargets).apply(compiler);
        compiler.hooks.afterCompile.tapPromise(plugin_name_1.PLUGIN_NAME, (compilation) => __awaiter(this, void 0, void 0, function* () {
            yield this.run(compilation);
        }));
    }
    run(compilation) {
        return __awaiter(this, void 0, void 0, function* () {
            if (compilation.name !== undefined || compilation.errors.length) {
                return;
            }
            const tempEmitter = new temp_emitter_1.TempEmitter(compilation);
            yield tempEmitter.init();
            const emitResult = yield tempEmitter.emit();
            const targetCompilerFactory = new babel_target_compiler_factory_1.BabelTargetCompilerFactory(compilation, this.compilationTargets, tempEmitter.context, emitResult, this.config, this.plugins, this.exclude);
            const childCompilers = this.targets
                .map((target) => targetCompilerFactory.createCompiler(target));
            try {
                yield this.runChildCompilers(compilation, childCompilers);
                this.cleanUpOriginalCompilation(compilation);
            }
            finally {
                yield tempEmitter.dispose();
            }
        });
    }
}
exports.TranspilerCompiler = TranspilerCompiler;
//# sourceMappingURL=transpiler.compiler.js.map